"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_tools_1 = require("graphql-tools");
const defaultHandlers_1 = require("../defaultHandlers");
exports.default = (overiddeHandler) => {
    const handler = overiddeHandler || defaultHandlers_1.hasRoleHandler;
    return class HasRoleDirective extends graphql_tools_1.SchemaDirectiveVisitor {
        visitFieldDefinition(field) {
            const { resolve } = field;
            const { roles } = this.args;
            field.resolve = async function (...args) {
                const ctx = args[2];
                handler(ctx, roles);
                return resolve.apply(this, args);
            };
        }
        visitInputFieldDefinition(field, details) {
            var _a, _b;
            const { name, defaultValue } = field;
            const { roles } = this.args;
            const mutations = Object.values(((_a = this.schema.getMutationType()) === null || _a === void 0 ? void 0 : _a.getFields()) || []);
            const queries = Object.values(((_b = this.schema.getQueryType()) === null || _b === void 0 ? void 0 : _b.getFields()) || []);
            mutations.filter(this.filterInputPred(details)).forEach(mutation => {
                const { resolve } = mutation;
                mutation.resolve = async (...args) => {
                    const params = args[1];
                    const subKey = Object.values(params).find(el => el && el[name]);
                    if (params[name] !== defaultValue || (subKey && subKey[name] !== defaultValue)) {
                        const ctx = args[2];
                        handler(ctx, roles);
                    }
                    return resolve.apply(this, args);
                };
            });
            queries.filter(this.filterInputPred(details)).forEach(query => {
                const { resolve } = query;
                query.resolve = async (...args) => {
                    const params = args[1];
                    const subKey = Object.values(params).find(el => el && el[name]);
                    if (params[name] !== defaultValue || (subKey && subKey[name] !== defaultValue)) {
                        const ctx = args[2];
                        handler(ctx, roles);
                    }
                    return resolve.apply(this, args);
                };
            });
        }
        filterInputPred(details) {
            return (mutation) => mutation.args.find(arg => arg.type.ofType === details.objectType);
        }
    };
};
